//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.assistant;

import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.input.RadioInput;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.util.ApplicationException;

/**
 * Der Start-Dialog zum Anlegen eines neues Kontos. Ermglicht die Auswahl der
 * Bank.
 * 
 * @author Frank Mnster
 *
 */
public class CreateKontoStartDialog extends AbstractDialog<String> implements MouseListener {

	/**
	 * RadioGroupID
	 */
	private final static String RADIO_GROUP_ID = "banks"; //$NON-NLS-1$

	/**
	 * Ausgewhlter RadioButton
	 */
	private transient String choosen;

	/**
	 */
	public CreateKontoStartDialog() {
		super(POSITION_CENTER);
		this.setTitle(I18N.tr("CreateKontoStartDialog.dialogTitle")); //$NON-NLS-1$
		this.setSize(550, 400);
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected void paint(final Composite parent) {

		// Erstelle einfachen Container
		final SimpleContainer container = new SimpleContainer(parent, true, 2);

		container.addText(" ", true); //$NON-NLS-1$
		container.addText(I18N.tr("CreateKontoStartDialog.chooseAccountText"), true); //$NON-NLS-1$
		final Set<String> blzs = MahoeSynchronizeBackend.BLZ_SUPPORTED.keySet();
		RadioInput input;
		for (final String blz : blzs) {
			input = new RadioInput(RADIO_GROUP_ID, blz); // NOPMD
			input.setName(MahoeSynchronizeBackend.BLZ_SUPPORTED.get(blz));
			container.addRadioInput(input, StringUtils.EMPTY);

			input.getControl().addMouseListener(this);

		}
		RadioInput.select(RADIO_GROUP_ID, blzs.iterator().next());

		final ButtonArea buttons = new ButtonArea();
		buttons.addButton(I18N.tr("CreateKontoStartDialog.startAssistentBtn"), new Action() { //$NON-NLS-1$
			/**
			 * 
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context) throws ApplicationException {
				choosen = (String) RadioInput.getValue(RADIO_GROUP_ID);
				close();
			}
		}, null, true, "document-save.png"); //$NON-NLS-1$
		// Abbrechen Button einfgen
		buttons.addButton(I18N.tr("CreateKontoStartDialog.cancelBtn"), new Action() { //$NON-NLS-1$
			/**
			 * 
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context) throws ApplicationException {
				throw new OperationCanceledException();
			}
		}, null, false, "process-stop.png"); //$NON-NLS-1$

		container.addButtonArea(buttons);
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
	 */
	@Override
	protected String getData() {
		return choosen;
	}

	/**
	 * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(org.eclipse.swt.events.MouseEvent)
	 */
	@Override
	public void mouseDoubleClick(final MouseEvent arg0) {
		// Angeklickten Button auswhlen und Dialog schlieen
		choosen = (String) RadioInput.getValue(RADIO_GROUP_ID);
		close();
	}

	/**
	 * @see org.eclipse.swt.events.MouseListener#mouseDown(org.eclipse.swt.events.MouseEvent)
	 */
	@Override
	public void mouseDown(final MouseEvent arg0) {
		// Hier machen wir nix
	}

	/**
	 * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
	 */
	@Override
	public void mouseUp(final MouseEvent arg0) {
		// Hier machen wir nix
	}

}