//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.assistant;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import de.frankmuenster.hibiscus.mahoe.bsh.assistant.BshAccountMigrator;
import de.frankmuenster.hibiscus.mahoe.santander.assistant.SantanderAccountMigrator;
import de.frankmuenster.hibiscus.mahoe.targobank.assistant.TargoBankAccountMigrator;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.messaging.Message;
import de.willuhn.jameica.messaging.MessageConsumer;
import de.willuhn.jameica.messaging.SystemMessage;
import de.willuhn.jameica.plugin.Plugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Started den Account-Migrator erst nach vollstndigem Systemstart
 * 
 * @author Frank Mnster
 * 
 */
public class MahoeUpdateHandler implements MessageConsumer {

	/**
	 * Liste mit allen bekannten AccountMigratoren/Updatern
	 */
	private transient final Set<Class<? extends AccountMigrator>> accountMigrators = new HashSet<Class<? extends AccountMigrator>>();

	/**
	 * Erstellt die Migrator-Klasse und registriert sich selbst in der
	 * MessagingFactory
	 */
	public MahoeUpdateHandler() {
		accountMigrators.add(TargoBankAccountMigrator.class);
		accountMigrators.add(BshAccountMigrator.class);
		accountMigrators.add(SantanderAccountMigrator.class);
	}

	/**
	 * @see de.willuhn.jameica.messaging.MessageConsumer#getExpectedMessageTypes()
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public Class[] getExpectedMessageTypes() {
		return new Class[] { SystemMessage.class };
	}

	/**
	 * @throws ApplicationException
	 * @see de.willuhn.jameica.messaging.MessageConsumer#handleMessage(de.willuhn.jameica.messaging.Message)
	 */
	@Override
	public void handleMessage(final Message message) throws ApplicationException {
		final SystemMessage sysMsg = (SystemMessage) message;
		if (sysMsg.getStatusCode() == SystemMessage.SYSTEM_STARTED) {

			Logger.debug("System gestartet. Nun kann der Migrator gestartet werden."); //$NON-NLS-1$

			checkOldPlugins();

			updateAccounts();

			// final HandlerThread handlerThread = new HandlerThread();
			// handlerThread.start();
			unregister();
		}

	}

	/**
	 * @see de.willuhn.jameica.messaging.MessageConsumer#autoRegister()
	 */
	@Override
	public boolean autoRegister() {
		return true;
	}

	private void unregister() {
		// Unregister
		Application.getMessagingFactory().unRegisterMessageConsumer(this);
	}

	/**
	 * @throws ApplicationException
	 * 
	 */
	private void checkOldPlugins() throws ApplicationException {

		final List<Plugin> plugins = Application.getPluginLoader().getInstalledPlugins();

		for (final Class<? extends AccountMigrator> clazz : accountMigrators) {

			try {
				final Constructor<? extends AccountMigrator> cons = clazz.getConstructor();
				final AccountMigrator accountMigrator = (AccountMigrator) cons
						.newInstance();
				for (final Plugin plugin : plugins) {
					accountMigrator.migrate(plugin);
				}
			} catch (NoSuchMethodException | SecurityException | InstantiationException | IllegalAccessException
					| IllegalArgumentException | InvocationTargetException e) {
				Logger.error(I18N.tr("MahoeUpdateHandler.checkforOldScriptingModulesFailed", e.getMessage()), e); //$NON-NLS-1$
			}
		}

	}

	private void updateAccounts() throws ApplicationException {

		for (final Class<? extends AccountMigrator> clazz : accountMigrators) {

			try {
				final Constructor<? extends AccountMigrator> cons = clazz.getConstructor();
				final AccountMigrator accountMigrator = (AccountMigrator) cons
						.newInstance();

				final HBCIDBService hcbiService = Settings.getDBService();
				final DBIterator<Konto> accounts = hcbiService.createList(Konto.class);
				while (accounts.hasNext()) {
					final Konto account = accounts.next();
					accountMigrator.update(account);
				}
			} catch (NoSuchMethodException | SecurityException | InstantiationException | IllegalAccessException
					| IllegalArgumentException | InvocationTargetException | RemoteException e) {
				Logger.error(I18N.tr("MahoeUpdateHandler.checkforOldScriptingModulesFailed", e.getMessage()), e); //$NON-NLS-1$
			}
		}

	}

}
