//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.assistant;

import java.rmi.RemoteException;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateAccountException;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateAccountAssistant;
import de.frankmuenster.hibiscus.mahoe.bsh.SchwaebischHall;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Account;
import de.frankmuenster.hibiscus.mahoe.bsh.web.BshWebSite;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.willuhn.jameica.hbci.HBCI;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * BSH Konto-Anlagen-Assistent
 * 
 * @author Frank Mnster
 * 
 */
public final class BshCreateAccountAssistent implements CreateAccountAssistant {

	/**
	 * Standardkonstruktor
	 */
	public BshCreateAccountAssistent() {
		// Absichtlich leer
	}

	/**
	 * @see CreateAccountAssistant#createAccount()
	 */
	@Override
	public Konto createAccount() throws CreateAccountException, OperationCanceledException {

		// Start Dialog zur KD-Nummer abfrage
		final BshCreateKontoStartDialog startDialog = new BshCreateKontoStartDialog();
		String kdNummer;
		try {
			kdNummer = startDialog.open();
		} catch (OperationCanceledException e) { // NOPMD
			throw e;
		} catch (Exception e) { // NOPMD
			Logger.warn(I18N.tr("BshCreateAccountAssistent.openDialogFailedLog")); //$NON-NLS-1$
			throw new CreateAccountException(I18N.tr("BshCreateAccountAssistent.openDialogFailed"), e); //$NON-NLS-1$
		}
		if (StringUtils.isEmpty(kdNummer)) {
			throw new CreateAccountException(I18N.tr("BshCreateAccountAssistent.kdNumberNotProvided")); //$NON-NLS-1$
		}

		List<Account> accounts = null;
		try {
			accounts = getAccounts(kdNummer);
		} catch (WebSiteException | ApplicationException e1) {
			Logger.error(e1.getMessage(), e1);
			throw new CreateAccountException(e1.getMessage(), e1);
		}

		final BshCreateAccountDialog infoDialog = new BshCreateAccountDialog(kdNummer, accounts);
		BshAccountInfo info = null;
		try {
			info = infoDialog.open();
		} catch (OperationCanceledException e) { // NOPMD
			throw e;
		} catch (Exception e) { // NOPMD
			Logger.warn(I18N.tr("BshCreateAccountAssistent.openDialogFailedLog")); //$NON-NLS-1$
			throw new CreateAccountException(I18N.tr("BshCreateAccountAssistent.openDialogFailed"), e); //$NON-NLS-1$
		}
		if (StringUtils.isEmpty(info.getBlzDz()) || StringUtils.isEmpty(info.getVtn())) {
			throw new CreateAccountException(I18N.tr("BshCreateAccountAssistent.kdNumberNotProvided")); //$NON-NLS-1$
		}
		return createKonto(info);

	}

	/**
	 * Ermittelt die Liste aller Vertrge der Kundennummer
	 * 
	 * @param kdNummer
	 * 
	 * @return die Liste aller Vertrge
	 * 
	 * @throws ApplicationException
	 * @throws WebSiteException
	 * @throws OperationCanceledException
	 */
	private List<Account> getAccounts(final String kdNummer) throws WebSiteException, OperationCanceledException,
			ApplicationException {

		final BshWebSite webSite = BshWebSite.getInstance(kdNummer);
		return webSite.getAccounts();
	}

	/**
	 * Erstellt das Konto.
	 * 
	 * @param info
	 * @return der neu erstelle Konto
	 * @throws CreateAccountException
	 */
	private Konto createKonto(final BshAccountInfo info) throws CreateAccountException {

		HBCIDBService dbService = null;
		try {
			dbService = (HBCIDBService) Application.getServiceFactory().lookup(HBCI.class, "database"); //$NON-NLS-1$
		} catch (Exception e) { // NOPMD
			Logger.error(I18N.tr("BshCreateAccountAssistent.noDbAccess"), e); //$NON-NLS-1$
			throw new CreateAccountException(I18N.tr("BshCreateAccountAssistent.noDbAccess"), e); //$NON-NLS-1$
		}
		Konto konto;
		try {
			konto = dbService.createObject(Konto.class, null);

			konto.setName(info.getOwner());
			konto.setBezeichnung(info.getDescription());

			konto.setKontonummer(info.getCoreId());
			konto.setBLZ(SchwaebischHall.BLZ);
			konto.setIban(getIban(info));

			konto.setKundennummer(info.getKdNummer());
			konto.setUnterkonto(info.getLetterNumber());

			konto.setAccountType(KontoType.BAUSPAR.getValue());
			konto.setFlags(Konto.FLAG_NONE);
			konto.setKommentar(info.getComment());

			konto.setBackendClass(MahoeSynchronizeBackend.BACKEND_NAME);
			konto.store();

			// Setze META Daten
			konto.setMeta(MahoeSynchronizeBackend.BSH_VTN, info.getVtn());
			// konto.setMeta(MahoeSynchronizeBackend.CARD_NUMBER,
			// StringUtils.EMPTY);
			// Kontoauszugebehandlung
			//konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF, "NEIN"); //$NON-NLS-1$
			// konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF_PATH,
			// StringUtils.EMPTY);

			// Sync-Options setzen
			final SynchronizeOptions sync = new SynchronizeOptions(konto);
			sync.setAutoSaldo(false);
			sync.setSyncAuslandsUeberweisungen(false);
			sync.setSyncKontoauszuege(true);
			sync.setSyncOffline(false);
			sync.setSyncSaldo(true);
			sync.setSyncSepaDauerauftraege(false);
			sync.setSyncSepaLastschriften(false);
		} catch (RemoteException | ApplicationException e) {
			Logger.error(I18N.tr("BshCreateAccountAssistent.createAccountFailedLog"), e); //$NON-NLS-1$
			throw new CreateAccountException(I18N.tr("BshCreateAccountAssistent.createAccountFailed"), e); //$NON-NLS-1$
		}

		return konto;

	}

	/**
	 * Ermittelt die IBAN.
	 * 
	 * @param info
	 * 
	 * @return die IBAN, oder NULL
	 */
	private String getIban(final BshAccountInfo info) {

		String iban = null;
		try {
			final BshWebSite webSite = BshWebSite.getInstance(info.getKdNummer());
			final String bsnKurz = info.getCoreId() + info.getLetterNumber();
			iban = webSite.getIban(bsnKurz, info.getBlzDz());
			if (iban != null) {
				// Evtl. Leerzeichen entfernen
				iban = iban.replaceAll(" ", StringUtils.EMPTY); //$NON-NLS-1$
			}
		} catch (WebSiteException | ApplicationException e) {
			// Nur Loggen
			Logger.error(e.getMessage(), e);
		}

		return iban;
	}
}
