//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json;

import java.math.BigDecimal;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.SimpleMoneyDeserializer;

/**
 * JSON Freistellungsbetrag Objekt
 * 
 * @author Frank Mnster
 * 
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "coreid", "usedamount", "anticipatedamount", "type", "suggestedamount", "recordedondate",
		"tariffamount", "validfromdate" })
public class Exemption {

	/**
	 * Erwartete Zinsen
	 */
	@JsonProperty("anticipatedamount")
	private transient BigDecimal anticipatedamount;

	/**
	 * Haupt-Vertrag-ID
	 */
	@JsonProperty("coreid")
	private transient String coreid;

	/**
	 * Erfasst am
	 */
	@JsonProperty("recordedondate")
	private transient Date recordedondate;

	/**
	 * Vorgeschlagener Betrag
	 */
	@JsonProperty("suggestedamount")
	private transient BigDecimal suggestedamount;

	/**
	 * Erfasster Betrag
	 */
	@JsonProperty("tariffamount")
	private transient BigDecimal tariffamount;

	/**
	 * Freistellungstyp ???
	 */
	@JsonProperty("type")
	private transient String type;

	/**
	 * Verbrauchter Betrag
	 */
	@JsonProperty("usedamount")
	private transient BigDecimal usedamount;

	/**
	 * G&auml;ltig ab
	 */
	@JsonProperty("validfromdate")
	private transient Date validfromdate;

	/**
	 * Standardkonstruktor
	 */
	public Exemption() {
		// Absichtlich leer
	}

	/**
	 * Liefert die erwarteten Zinsen
	 * 
	 * @return die erwarteten Zinsen
	 */
	@JsonProperty("anticipatedamount")
	@JsonDeserialize(using = SimpleMoneyDeserializer.class)
	public BigDecimal getAnticipatedamount() {
		return anticipatedamount;
	}

	/**
	 * Liefert die Vertagshaupt-ID
	 * 
	 * @return die Vertagshaupt-ID
	 */
	@JsonProperty("coreid")
	public String getCoreid() {
		return coreid;
	}

	/**
	 * Liefert das Erfassungsdatum
	 * 
	 * @return das Erfassungsdatum
	 */
	@JsonProperty("recordedondate")
	@JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "dd.MM.yyyy", timezone = "CET")
	public Date getRecordedondate() {
		return recordedondate;
	}

	/**
	 * Liefert den vorgeschlagenen Betrag
	 * 
	 * @return den vorgeschlagenen Betrag
	 */
	@JsonProperty("suggestedamount")
	@JsonDeserialize(using = SimpleMoneyDeserializer.class)
	public BigDecimal getSuggestedamount() {
		return suggestedamount;
	}

	/**
	 * Liefert den erfassten Betrag
	 * 
	 * @return den erfassten Betrag
	 */
	@JsonProperty("tariffamount")
	@JsonDeserialize(using = SimpleMoneyDeserializer.class)
	public BigDecimal getTariffamount() {
		return tariffamount;
	}

	/**
	 * Liefert den Freistellungstyp
	 * 
	 * @return den Freistellungstyp
	 */
	@JsonProperty("type")
	public String getType() {
		return type;
	}

	/**
	 * Liefert den verbrauchten Betrag
	 * 
	 * @return den verbrauchten Betrag
	 */
	@JsonProperty("usedamount")
	@JsonDeserialize(using = SimpleMoneyDeserializer.class)
	public BigDecimal getUsedamount() {
		return usedamount;
	}

	/**
	 * Liefert das Datum, ab dem der Freistellungsauftrag gltig ist
	 * 
	 * @return das Datum, ab dem der Freistellungsauftrag gltig ist
	 */
	@JsonProperty("validfromdate")
	@JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "dd.MM.yyyy", timezone = "CET")
	public Date getValidfromdate() {
		return validfromdate;
	}

}
