//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json;

import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * JSON Nachricht Objekt
 * 
 * @author Frank Mnster
 * 
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "id", "date", "title", "type", "read", "text", "deletable", "nbaurl" })
public class Message {

	/**
	 * Nachrichten ID
	 */
	@JsonProperty("id")
	private transient String id; // NOPMD

	/**
	 * Datum der Nachricht
	 */
	@JsonProperty("date")
	@JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "dd.MM.yyyy", timezone = "CET")
	private transient Date date;

	/**
	 * Titel/Subject der Nachricht
	 */
	@JsonProperty("title")
	private transient String title;

	/**
	 * Der Type der Nachricht
	 */
	@JsonProperty("type")
	private transient String type;

	/**
	 * Nachricht schon gelsen
	 */
	@JsonProperty("read")
	private transient Boolean read;

	/**
	 * Nachrichtentext
	 */
	@JsonProperty("text")
	private transient String text;

	/**
	 * Kann die Nachricht gelsht werden
	 */
	@JsonProperty("deletable")
	private transient Boolean deletable;

	/**
	 * Werbe-URL
	 */
	@JsonProperty("nbaurl")
	private transient String nbaurl;

	/**
	 * Standardkonstruktor
	 */
	public Message() {
		// Absichtlich leer
	}

	/**
	 * Liefert die ID
	 * 
	 * @return die ID
	 */
	@JsonProperty("id")
	public String getId() {
		return id;
	}

	/**
	 * Liefert das Datum
	 * 
	 * @return das Datum
	 */
	@JsonProperty("date")
	public Date getDate() {
		return date;
	}

	/**
	 * Liefert den Titel
	 * 
	 * @return den Titel
	 */
	@JsonProperty("title")
	public String getTitle() {
		return title;
	}

	/**
	 * Liefert den Typ
	 * 
	 * @return den Typ
	 */
	@JsonProperty("type")
	public String getType() {
		return type;
	}

	/**
	 * Nachricht schon gelesen.
	 * 
	 * @return <i>true</i>, falls die Nachricht schon gelesen ist, <i>false</i>
	 *         sonst.
	 */
	@JsonProperty("read")
	public Boolean isRead() {
		return read;
	}

	/**
	 * Liefert den Text der Nachricht.
	 * 
	 * @return den Text
	 */
	@JsonProperty("text")
	public String getText() {
		return text;
	}

	/**
	 * Kann die Nachricht gelscht werden.
	 * 
	 * @return <i>true</i>, falls die Nachricht gelscht werden kann,
	 *         <i>false</i> sonst.
	 */
	@JsonProperty("deletable")
	public Boolean isDeletable() {
		return deletable;
	}

	/**
	 * Liefert die Werbe-URL. Wir hier wohl nicht benutzt.
	 * 
	 * @return die Werbe-URL
	 */
	@JsonProperty("nbaurl")
	public String getNbaurl() {
		return nbaurl;
	}

}
