//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.MoneyDeserializer;
import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.SimpleMoneyDeserializer;

/**
 * JSON Vertrags Objekt
 * 
 * @author Frank Mnster
 * 
 */
@JsonIgnoreProperties(ignoreUnknown = true)
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "tarif", "savesum", "startDate", "chooseassignment", "moreassignment", "bonusamount",
		"currentbalance", "savinginterestrate", "ratingnumber", "grantinterestamount", "grantremaining", "grantend" })
public class Contract {

	/**
	 * Der Bonus-Bbetrag
	 */
	@JsonProperty("bonusamount")
	private transient String bonusAmount;

	/**
	 * Wahl-, Mehrzuteilung in %
	 */
	@JsonProperty("chooseassignment")
	private transient String chooseAssignment;

	/**
	 * Lfd. Saldo
	 */
	@JsonProperty("currentbalance")
	@JsonDeserialize(using = MoneyDeserializer.class)
	private transient BigDecimal currentBalance;

	/**
	 * Weitere Zuteilungen ???
	 */
	@JsonProperty("moreassignment")
	private transient String moreAssignment;

	/**
	 * Bewertungszahl
	 */
	@JsonProperty("ratingnumber")
	private transient String ratingNumber;

	/**
	 * Bausparsumme
	 */
	@JsonProperty("savesum")
	@JsonDeserialize(using = SimpleMoneyDeserializer.class)
	private transient BigDecimal saveSum;

	/**
	 * Sparzins
	 */
	@JsonProperty("savinginterestrate")
	private transient String savingInterestRate; // NOPMD

	/**
	 * Vertragsbeginn
	 */
	@JsonProperty("startdate")
	@JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "dd.MM.yyyy", timezone = "CET")
	private transient Date startDate;

	/**
	 * Tarif
	 */
	@JsonProperty("tarif")
	private transient String tarif;

	/**
	 * H&ouml;he der Rate
	 */
	@JsonProperty("grantinterestamount")
	@JsonDeserialize(using = MoneyDeserializer.class)
	private transient BigDecimal grantInterestAmount; // NOPMD

	/**
	 * Restschuld
	 */
	@JsonProperty("grantremaining")
	@JsonDeserialize(using = MoneyDeserializer.class)
	private transient BigDecimal grantRemaining; // NOPMD

	/**
	 * Voraussichtliches Vertragsende
	 */
	@JsonProperty("grantend")
	private transient String grantend;

	/**
	 * Standardkonstruktor
	 */
	public Contract() {
		// Absichtlich leer
	}

	/**
	 * Liefert den Bouns-Betrag
	 * 
	 * @return den Bouns-Betrag
	 */
	@JsonProperty("bonusamount")
	public String getBonusamount() {
		return this.bonusAmount;
	}

	/**
	 * Liefert die Wahl-, Mehrzuteilung in %
	 * 
	 * @return die Wahl-, Mehrzuteilung in %
	 */
	@JsonProperty("chooseassignment")
	public String getChooseassignment() {
		return this.chooseAssignment;
	}

	/**
	 * Liefert das lfd. Saldo
	 * 
	 * @return das lfd. Saldo
	 */
	@JsonProperty("currentbalance")
	public BigDecimal getCurrentbalance() {
		return this.currentBalance;
	}

	/**
	 * Liefert das lfd. Saldo.
	 * 
	 * @return das lfd. Saldo.
	 */
	@JsonProperty("moreassignment")
	public String getMoreAssignment() {
		return this.moreAssignment;
	}

	/**
	 * Liefert die Bewertungszahl.
	 * 
	 * @return die Bewertungszahl.
	 */
	@JsonProperty("ratingnumber")
	public String getRatingNumber() {
		return this.ratingNumber;
	}

	/**
	 * Liefert die Bausparsumme
	 * 
	 * @return die Bausparsumme
	 */

	public BigDecimal getSaveSum() {
		return this.saveSum;
	}

	/**
	 * Liefert den Sparzins
	 * 
	 * @return den Sparzins
	 */
	@JsonProperty("savinginterestrate")
	public String getSavingInterestRate() {
		return this.savingInterestRate;
	}

	/**
	 * Liefert den Vertragsbeginn
	 * 
	 * @return den Vertragsbeginn
	 */
	@JsonProperty("startdate")
	public Date getStartDate() {
		return this.startDate;
	}

	/**
	 * Liefert den Tarif
	 * 
	 * @return den Tarif
	 */
	public String getTarif() {
		return this.tarif;
	}

	/**
	 * Liefert die H&ouml;he der Rate
	 * 
	 * @return die H&ouml;he der Rate
	 */

	public BigDecimal getGrantInterestAmount() {
		return this.grantInterestAmount;
	}

	/**
	 * Liefert die Restschuld
	 * 
	 * @return die Restschuld
	 */

	public BigDecimal getGrantRemaining() {
		return this.grantRemaining;
	}

	/**
	 * Liefert das voraussichtliches Vertragsende
	 * 
	 * @return das voraussichtliches Vertragsende
	 */
	public String getGrantend() {
		return this.grantend;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return String
				.format("Contract [bonusAmount=%s, chooseAssignment=%s, currentBalance=%s, moreAssignment=%s, ratingNumber=%s, saveSum=%s, savingInterestRate=%s, startDate=%s, tarif=%s]", //$NON-NLS-1$
						this.bonusAmount, this.chooseAssignment, this.currentBalance, this.moreAssignment,
						this.ratingNumber, this.saveSum, this.savingInterestRate, this.startDate, this.tarif);
	}
}
