//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.MoneyDeserializer;

/**
 * Verteragsdetails - Debit
 * 
 * @author Frank Mnster
 * 
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "suggestedamount", "directdebits" })
public class Debit {

	/**
	 * Der Regelsparbetrag
	 */
	@JsonProperty("suggestedamount")
	@JsonDeserialize(using = MoneyDeserializer.class)
	private transient BigDecimal suggestedAmount;

	/**
	 * Liste mit allen direkten Einzgen
	 */
	@JsonProperty("directdebits")
	private transient List<DirectDebit> directDebits;

	/**
	 * Standardkonstruktor
	 */
	public Debit() {
		// Absichtlich leer
	}

	/**
	 * Liefert den Regelsparbetrag
	 * 
	 * @return den Regelsparbetrag
	 */
	@JsonProperty("suggestedamount")
	public BigDecimal getSuggestedAmount() {
		return suggestedAmount;
	}

	/**
	 * Liefert die Liste mit allen direkten Einzgen
	 * 
	 * @return die Liste mit allen direkten Einzgen
	 */
	@JsonProperty("directdebits")
	public List<DirectDebit> getDirectDebits() {
		return directDebits;
	}

}
