//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.SimpleMoneyDeserializer;

/**
 * Vertragdetails - Subsidy
 * 
 * @author Frank Mnster
 * 
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "sum", "details" })
public class Subsidy {

	/**
	 * Die Summe aller Prmien
	 */
	@JsonProperty("sum")
	@JsonDeserialize(using = SimpleMoneyDeserializer.class)
	private transient BigDecimal sum;

	/**
	 * Liste mit den Details der Prmien
	 */
	@JsonProperty("details")
	private transient List<SubsidyDetail> details;

	/**
	 * Standardkonstruktor
	 */
	public Subsidy() {
		// Absichtlich leer
	}

	/**
	 * Liefert die Summe aller Prmien.
	 * 
	 * @return die Summe aller Prmien.
	 */
	@JsonProperty("sum")
	public BigDecimal getSum() {
		return sum;
	}

	/**
	 * Liefert die Liste mit den Details der Prmien.
	 * 
	 * @return die Liste mit den Details.
	 */
	@JsonProperty("details")
	public List<SubsidyDetail> getDetails() {
		return details;
	}

}
