//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonNode;

import de.frankmuenster.hibiscus.mahoe.bsh.json.Exemption;

/**
 * Deserializer fr die Freistellungsauftrags-Liste. Im Prinzip hebt sie die
 * Liste eine Ebene hher und umgeht so das "data" JSON Object
 * 
 * @author Frank Mnster
 * 
 */
public class ExemptionsDeserializer extends JsonDeserializer<List<Exemption>> {

	/**
	 * Contracts JSON identifier
	 */
	private static final String EXEMPTIONS = "exemptions"; //$NON-NLS-1$

	/**
	 * Standardkonstruktor
	 */
	public ExemptionsDeserializer() {
		super();
	}

	/**
	 * @see com.fasterxml.jackson.databind.JsonDeserializer#deserialize(com.fasterxml.jackson.core.JsonParser,
	 *      com.fasterxml.jackson.databind.DeserializationContext)
	 */
	@Override
	public List<Exemption> deserialize(final JsonParser jsonParser, final DeserializationContext context)
			throws IOException, JsonProcessingException {
		List<Exemption> exemptionsList = new ArrayList<>();

		final ObjectCodec objectCodec = jsonParser.getCodec();
		final JsonNode jsonNode = objectCodec.readTree(jsonParser);

		final Iterator<Map.Entry<String, JsonNode>> fields = jsonNode.fields();
		while (fields.hasNext()) {
			final Map.Entry<String, JsonNode> field = fields.next();
			final String fieldName = field.getKey();
			if (EXEMPTIONS.equals(fieldName)) {
				final JsonNode fieldValue = field.getValue();
				if (!fieldValue.isObject()) {
					final Exemption[] exemptions = objectCodec.treeToValue(fieldValue, Exemption[].class);
					exemptionsList = Arrays.asList(exemptions);
					break;
				}
			}
		}

		return exemptionsList;
	}
}
