//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer;

import java.io.IOException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParsePosition;
import java.util.Locale;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;

/**
 * Deserializer fr die Geld-Betrge ohne Vorzeichen der BSH
 * 
 * @author Frank Mnster
 * 
 */
public class SimpleMoneyDeserializer extends JsonDeserializer<BigDecimal> {

	/**
	 * Geldbetragsformat der BSH
	 */
	private static final String PATTERN = "#,##0.00"; //$NON-NLS-1$

	/**
	 * Standardkonstruktor
	 */
	public SimpleMoneyDeserializer() {
		super();
	}

	/**
	 * @see JsonDeserializer#deserialize(JsonParser, DeserializationContext)
	 */
	@Override
	public BigDecimal deserialize(final JsonParser jsonParser, final DeserializationContext arg1) throws IOException,
			JsonProcessingException {
		final NumberFormat numberFormat = NumberFormat.getNumberInstance(Locale.GERMAN);
		final DecimalFormat format = (DecimalFormat) numberFormat;
		format.applyPattern(PATTERN);
		format.setParseBigDecimal(true);

		final String textValue = jsonParser.getText();
		return (BigDecimal) format.parse(textValue, new ParsePosition(0));
	}

}
