//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.hibiscus.mahoe.gui.controller;

import java.rmi.RemoteException;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Event;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.bsh.SchwaebischHall;
import de.frankmuenster.hibiscus.mahoe.bsh.assistant.BshAccountInfo;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public class BshSettingsController extends AbstractSettingsController implements SettingsController {

    /**
     * Soll jedes mal nach dem Passwort gefragt werden anlegen
     */
    private transient CheckboxInput alwaysAskForPW;

    /**
     * Kontoinformationen
     */
    private final transient BshAccountInfo ktoInfo = new BshAccountInfo();

    /**
     * Standardkonstruktor
     */
    public BshSettingsController() {
        super(SchwaebischHall.BLZ);
    }

    /**
     * Liefert eine Checkbox zur Auswahl, ob das Passwort jedes mal Abgefragt werden soll.
     *
     * @return Ein CheckBoxInput Passwort jedes mal nachfragen.
     */
    public CheckboxInput getAlwaysAskForPW() {
        if (this.alwaysAskForPW == null) {
            Logger.debug("alwaysAskForPW: " + this.ktoInfo.isAlwaysAskForPW()); //$NON-NLS-1$
            this.alwaysAskForPW = new CheckboxInput(this.ktoInfo.isAlwaysAskForPW());
        }
        return this.alwaysAskForPW;
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.gui.controller.SettingsController#speichern()
     */
    @Override
    public void speichern() {
        final Konto konto = (Konto) this.getKonto().getValue();
        this.ktoInfo.setAlwaysAskForPW((boolean) this.getAlwaysAskForPW().getValue());
        try {
            // Daten speichern
            final String alwaysAsk = this.ktoInfo.isAlwaysAskForPW() ? SettingsController.TEXT_JA
                    : SettingsController.TEXT_NEIN;
            konto.setMeta(MahoeSynchronizeBackend.PW_ASK_ALWAYS, alwaysAsk);
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("SantanderSettingsController.saveSettingsFailedLog"), e); //$NON-NLS-1$
            final String msg = I18N.tr("SantanderSettingsController.saveSettingsFailedMsg"); //$NON-NLS-1$
            final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
            Application.getMessagingFactory().sendMessage(msgUser);
        }
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.gui.controller.AbstractSettingsController#handleAccountChanged(org.eclipse.swt.widgets.Event)
     */
    @Override
    protected void handleAccountChanged(final Event event) {
        // Kontotyp ermitteln
        final Konto konto = (Konto) this.account.getValue();
        this.ktoInfo.setComment(StringUtils.EMPTY);
        this.ktoInfo.setDescription(StringUtils.EMPTY);
        this.ktoInfo.setKdNummer(StringUtils.EMPTY);
        this.ktoInfo.setOwner(StringUtils.EMPTY);
        this.ktoInfo.setSaveFolder(StringUtils.EMPTY);
        this.ktoInfo.setSavePDF(false);
        try {
            if (konto != null && SettingsController.TEXT_JA
                    .equals(konto.getMeta(MahoeSynchronizeBackend.PW_ASK_ALWAYS, SettingsController.TEXT_NEIN)
                            .toUpperCase(Locale.GERMAN))) {
                this.ktoInfo.setAlwaysAskForPW(true);
                this.getAlwaysAskForPW().setValue(true);
            } else {
                this.ktoInfo.setAlwaysAskForPW(false);
                this.getAlwaysAskForPW().setValue(false);
            }
        } catch (final RemoteException e) {
            final String msg = I18N.tr("BshSettingsController.readAccountSettingsFailed"); //$NON-NLS-1$
            Logger.error(msg, e);
            final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
            Application.getMessagingFactory().sendMessage(msgUser);
        }
    }
}
