package de.frankmuenster.hibiscus.mahoe.gui.controller;

import java.rmi.RemoteException;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.santander.SantanderBank;
import de.frankmuenster.hibiscus.mahoe.santander.assistant.SantanderCreateInfo;
import de.frankmuenster.hibiscus.mahoe.santander.web.SantanderBankEnum;
import de.frankmuenster.hibiscus.mahoe.utils.DebitCardValidator;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.DirectoryInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.hbci.gui.filter.KontoFilter;
import de.willuhn.jameica.hbci.gui.input.KontoInput;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJobSepaUeberweisung;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Kontroller fr die Settings View
 *
 * @author Frank Mnster
 */
@SuppressWarnings("PMD.GodClass")
public class SantanderSettingsController extends AbstractSettingsController implements SettingsController {

	/**
	 * Kontoinformationen
	 */
	private final transient SantanderCreateInfo ktoInfo = new SantanderCreateInfo();

	/**
	 * Banktyp (Consumer/Direkt)
	 */
	private transient TextInput bankTyp;

	/**
	 * Kontotyp
	 */
	private transient TextInput ktoType;

	/**
	 * Kreditkarten Name/Anbieter
	 */
	private transient TextInput cardNumber;

	/**
	 * Kreditkarten-Kontoauszge speichern
	 */
	private transient CheckboxInput savePDF;

	/**
	 * Speicherpfad
	 */
	private transient DirectoryInput savePdfFolder;

	/**
	 * Ausgleichsbuchung anlegen
	 */
	private transient CheckboxInput createClearing;

	/**
	 * Abbuchungsungskonto der Ausgleichsbuchung
	 */
	private transient KontoInput clearingAccount;

	/**
	 * Erstellt eine neue Instanz des Kontrollers
	 */
	public SantanderSettingsController() {
		super(SantanderBank.BLZ);
		// Kontoinformationen setzten
		this.ktoInfo.setBankType(null);
		this.ktoInfo.setComment(StringUtils.EMPTY);
		this.ktoInfo.setDescription(StringUtils.EMPTY);
		this.ktoInfo.setKtoNummer(StringUtils.EMPTY);
		this.ktoInfo.setKtoType(null);
		this.ktoInfo.setLogin(StringUtils.EMPTY);
		this.ktoInfo.setOwner(StringUtils.EMPTY);
		this.ktoInfo.setSavePDF(false);
		this.ktoInfo.setSaveFolder(StringUtils.EMPTY);
		this.ktoInfo.setCreateClearing(false);
		this.ktoInfo.setClearingAccount(null);
		this.ktoInfo.setOwner(StringUtils.EMPTY);
	}

	/**
	 * Liefert ein SelectInput mit der Auswahl des Banktyps
	 *
	 * @return ein SelectInput mit der Auswahl des Banktyps
	 */
	public TextInput getBankTyp() {
		if (this.bankTyp == null) {
			this.bankTyp = new TextInput(StringUtils.EMPTY);
			this.bankTyp.setEnabled(false);
			if (this.ktoInfo.getBankType() != null) {
				this.bankTyp.setValue(this.ktoInfo.getBankType().getBankName());
			}
		}
		return this.bankTyp;
	}

	/**
	 * Liefert ein SelectInput mit der Auswahl des Banktyps
	 *
	 * @return ein SelectInput mit der Auswahl des Banktyps
	 */
	public TextInput getKontoTyp() {
		if (this.ktoType == null) {
			this.ktoType = new TextInput(StringUtils.EMPTY);
			this.ktoType.setEnabled(false);
			if (this.ktoInfo.getKtoType() != null) {
				this.ktoType.setValue(this.ktoInfo.getKtoType().getName());
			}
		}
		return this.ktoType;
	}

	/**
	 * Liefert ein TextInput f&uuml;r den Kartenname/anbieter
	 *
	 * @return ein TextInput f&uuml;r den Kartenname/anbieter
	 */
	public TextInput getCardNumber() {
		if (this.cardNumber == null) {
			this.cardNumber = new TextInput(StringUtils.EMPTY);
			if (StringUtils.isNotEmpty(this.ktoInfo.getKtoNummer())) {
				this.cardNumber.setValue(this.ktoInfo.getKtoNummer());
			}
			// Kartenname als Kommentar
			if (StringUtils.isNotEmpty(this.ktoInfo.getOwner())) {
				this.cardNumber.setComment(this.ktoInfo.getOwner());
			} else {
				this.cardNumber.setComment(StringUtils.EMPTY);
			}
		}
		return this.cardNumber;
	}

	/**
	 * Liefert eine Checkbox zur Auswahl, ob die Kreditkartenauszge gespeichert
	 * werden sollen.
	 *
	 * @return Ein CheckBoxInput zur Auswahl, ob die Kreditkartenauszge
	 *         gespeichert werden sollen.
	 */
	public CheckboxInput getSavePDF() {
		if (this.savePDF == null) {
			Logger.debug("savePDF: " + this.ktoInfo.isSavePDF()); //$NON-NLS-1$
			this.savePDF = new CheckboxInput(this.ktoInfo.isSavePDF());
			this.savePDF.addListener(new Listener() {

				/**
				 * @see Listener#handleEvent(Event)
				 */
				@Override
				public void handleEvent(final Event event) {
					SantanderSettingsController.this.handleSaveSantanderPDFEvent();
				}
			});
		}
		return this.savePDF;
	}

	/**
	 * Liefert ein Kontoauswahlfeld
	 *
	 * @return Kontoauswahlfeld
	 */
	public DirectoryInput getSavePdfFolder() {
		if (this.savePdfFolder == null) {
			this.savePdfFolder = new DirectoryInput(this.ktoInfo.getSaveFolder());
		}
		if (this.ktoInfo.isSavePDF()) {
			this.savePdfFolder.enable();
		} else {
			this.savePdfFolder.disable();
		}
		return this.savePdfFolder;
	}

	/**
	 * Liefert eine Checkbox zur Auswahl, ob die Kreditkartenauszge gespeichert
	 * werden sollen.
	 *
	 * @return Ein CheckBoxInput zur Auswahl, ob die Kreditkartenauszge
	 *         gespeichert werden sollen.
	 */
	public CheckboxInput getCreateClearing() {
		if (this.createClearing == null) {
			Logger.debug("createClearing: " + this.ktoInfo.isCreateClearing()); //$NON-NLS-1$
			this.createClearing = new CheckboxInput(this.ktoInfo.isCreateClearing());
			this.createClearing.addListener(new Listener() {

				/**
				 * @see Listener#handleEvent(Event)
				 */
				@Override
				public void handleEvent(final Event event) {
					SantanderSettingsController.this.handleCreateTransaction();
				}
			});
		}
		return this.createClearing;
	}

	/**
	 * Liefert ein Kontoauswahlfeld
	 *
	 * @return Kontoauswahlfeld
	 */
	public KontoInput getClearingAccount() {
		if (this.clearingAccount == null) {
			final Konto konto = this.ktoInfo.getClearingAccount();
			try {
				final KontoFilter ktoFilter = KontoFilter.createForeign(SynchronizeJobSepaUeberweisung.class);
				this.clearingAccount = new KontoInput(konto, ktoFilter);
			} catch (final RemoteException e) {
				Logger.error(I18N.tr("SantanderSettingsController.createAccountListFailed"), e); //$NON-NLS-1$
			}
		}
		if (this.ktoInfo.isCreateClearing()) {
			this.clearingAccount.enable();
		} else {
			this.clearingAccount.disable();
		}
		return this.clearingAccount;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.gui.controller.SettingsController#speichern
	 *      ()
	 */
	@Override
	public void speichern() {
		this.ktoInfo.setSavePDF((Boolean) this.savePDF.getValue());
		this.ktoInfo.setSaveFolder((String) this.savePdfFolder.getValue());
		final Boolean testsOK = this.validateData();
		if (testsOK) {
			final Konto konto = (Konto) this.getKonto().getValue();
			try {
				// Daten speichern
				if (this.ktoInfo.getKtoType().equals(KontoType.GIRO)) {
					konto.setMeta(MahoeSynchronizeBackend.CARD_NUMBER, StringUtils.EMPTY);
					konto.setMeta(MahoeSynchronizeBackend.CARD_COMPANY, StringUtils.EMPTY);
					// Ausgleichbuchung
					konto.setMeta(MahoeSynchronizeBackend.CREATE_CLEARING, SantanderSettingsController.TEXT_NEIN);
					konto.setMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT, StringUtils.EMPTY);
				} else {
					konto.setMeta(MahoeSynchronizeBackend.CARD_NUMBER, this.ktoInfo.getKtoNummer());
					// konto.setMeta(MahoeSynchronizeBackend.CARD_COMPANY,
					// StringUtils.EMPTY);
					// Ausgleichbuchung
					final String createClearing = this.ktoInfo.isCreateClearing() ? SantanderSettingsController.TEXT_JA
							: SantanderSettingsController.TEXT_NEIN;
					konto.setMeta(MahoeSynchronizeBackend.CREATE_CLEARING, createClearing);
					konto.setMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT, this.ktoInfo.getClearingAccount().getID());
				}
				// Kontoauszugebehandlung
				final String savePdf = this.ktoInfo.isSavePDF() ? SantanderSettingsController.TEXT_JA
						: SantanderSettingsController.TEXT_NEIN;
				konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF, savePdf);
				konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF_PATH, this.ktoInfo.getSaveFolder());
				final String msg = I18N.tr("SantanderSettingsController.settingsSaved"); //$NON-NLS-1$
				Logger.info(msg);
				final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_INFO);
				Application.getMessagingFactory().sendMessage(msgUser);
			} catch (final RemoteException e) {
				Logger.error(I18N.tr("SantanderSettingsController.saveSettingsFailedLog"), e); //$NON-NLS-1$
				final String msg = I18N.tr("SantanderSettingsController.saveSettingsFailedMsg"); //$NON-NLS-1$
				final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
				Application.getMessagingFactory().sendMessage(msgUser);
			}
		}
	}

	/**
	 * Validiert die Eingaben.
	 *
	 * @return <i>true</i> wenn alle ntigen Eingaben vorhanden und korrekt
	 *         sind, <i>false</i> sonst.
	 */
	private Boolean validateData() {
		boolean testsOK = true;
		if (KontoType.KREDITKARTE.equals(this.ktoInfo.getKtoType())) {
			this.ktoInfo.setCreateClearing((Boolean) this.createClearing.getValue());
			this.ktoInfo.setClearingAccount((Konto) this.clearingAccount.getValue());
			this.ktoInfo.setKtoNummer((String) this.cardNumber.getValue());
			// Tests
			if (!DebitCardValidator.isValidateCardNumber(this.ktoInfo.getKtoNummer())) {
				final String msg = I18N.tr("SantanderSettingsController.wrongCreditCardNumber"); //$NON-NLS-1$
				final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
				Application.getMessagingFactory().sendMessage(msgUser);
				testsOK = false;
			}
			if (this.ktoInfo.isCreateClearing() && !(this.ktoInfo.getClearingAccount() instanceof Konto)) {
				final String msg = I18N.tr("SantanderSettingsController.provideClearingAccount"); //$NON-NLS-1$
				final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
				Application.getMessagingFactory().sendMessage(msgUser);
				testsOK = false;
			}
		}
		return testsOK;
	}

	/**
	 */
	private void handleSaveSantanderPDFEvent() {
		if ((boolean) this.getSavePDF().getValue()) {
			Logger.debug("Enable savePdfFolder Fields"); //$NON-NLS-1$
			this.getSavePdfFolder().enable();
		} else {
			Logger.debug("Disable savePdfFolder Fields"); //$NON-NLS-1$
			this.getSavePdfFolder().disable();
		}
	}

	/**
	 */
	private void handleCreateTransaction() {
		if ((boolean) this.getCreateClearing().getValue()) {
			Logger.debug("Enable clearingAccount Fields"); //$NON-NLS-1$
			this.getClearingAccount().enable();
		} else {
			Logger.debug("Disable clearingAccount Fields"); //$NON-NLS-1$
			this.getClearingAccount().disable();
		}
	}

	/**
	 * @param event
	 */
	@Override
	protected void handleAccountChanged(final Event event) {
		// Kontotyp ermitteln
		final Konto konto = (Konto) this.account.getValue();
		// Not used fields
		this.ktoInfo.setComment(StringUtils.EMPTY);
		this.ktoInfo.setDescription(StringUtils.EMPTY);
		this.ktoInfo.setLogin(StringUtils.EMPTY);
		this.ktoInfo.setOwner(StringUtils.EMPTY);
		if (konto == null) {
			this.ktoInfo.setBankType(null);
			this.ktoInfo.setKtoType(KontoType.SONSTIGE);
			this.ktoInfo.setKtoNummer(StringUtils.EMPTY);
		} else {
			try {
				// Kontoinformationen setzten
				this.ktoInfo.setBankType(SantanderBankEnum.find(konto));
				this.ktoInfo.setKtoNummer(konto.getMeta(MahoeSynchronizeBackend.CARD_NUMBER, StringUtils.EMPTY));
				this.ktoInfo.setKtoType(KontoType.find(konto.getAccountType()));
				this.ktoInfo.setOwner(konto.getMeta(MahoeSynchronizeBackend.CARD_COMPANY, StringUtils.EMPTY));
				this.ktoInfo.setSaveFolder(konto.getMeta(MahoeSynchronizeBackend.SAVE_PDF_PATH, StringUtils.EMPTY));
				if (SantanderSettingsController.TEXT_JA
						.equals(konto.getMeta(MahoeSynchronizeBackend.SAVE_PDF, SantanderSettingsController.TEXT_NEIN)
								.toUpperCase(Locale.GERMAN))) {
					this.ktoInfo.setSavePDF(true);
				} else {
					this.ktoInfo.setSavePDF(false);
				}
				final Konto clearingAccount = Utils
						.getKonto(konto.getMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT, StringUtils.EMPTY));
				this.ktoInfo.setClearingAccount(clearingAccount);
				if (SantanderSettingsController.TEXT_JA.equals(
						konto.getMeta(MahoeSynchronizeBackend.CREATE_CLEARING, SantanderSettingsController.TEXT_NEIN)
								.toUpperCase(Locale.GERMAN))) {
					this.ktoInfo.setCreateClearing(true);
				} else {
					this.ktoInfo.setCreateClearing(false);
				}
			} catch (final RemoteException e) {
				final String msg = I18N.tr("SantanderSettingsController.readAccountSettingsFailed"); //$NON-NLS-1$
				Logger.error(msg, e);
				final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
				Application.getMessagingFactory().sendMessage(msgUser);
			}
		}
		if (this.ktoInfo.getBankType() == null) {
			this.getBankTyp().setValue(StringUtils.EMPTY);
		} else {
			this.getBankTyp().setValue(this.ktoInfo.getBankType().getBankName());
		}
		this.getKontoTyp().setValue(this.ktoInfo.getKtoType().getName());
		this.getSavePDF().setValue(this.ktoInfo.isSavePDF());
		this.getSavePdfFolder().setValue(this.ktoInfo.getSaveFolder());
		this.getCreateClearing().setValue(this.ktoInfo.isCreateClearing());
		this.getClearingAccount().setValue(this.ktoInfo.getClearingAccount());
		switch (this.ktoInfo.getKtoType()) {
		case GIRO:
			this.getCardNumber().disable();
			this.getSavePDF().enable();
			this.handleSaveSantanderPDFEvent();
			this.getCreateClearing().disable();
			this.handleCreateTransaction();
			break;
		case KREDITKARTE:
			this.getCardNumber().enable();
			this.getCardNumber().setValue(this.ktoInfo.getKtoNummer());
			this.getCardNumber().setComment(this.ktoInfo.getOwner());
			this.getSavePDF().enable();
			this.handleSaveSantanderPDFEvent();
			this.getCreateClearing().enable();
			this.handleCreateTransaction();
			break;
		default:
			this.disableAll();
			break;
		}
	}

	/**
	 *
	 */
	private void disableAll() {
		this.getSavePDF().disable();
		this.getSavePdfFolder().disable();
		this.getCardNumber().disable();
		this.getCreateClearing().disable();
		this.getClearingAccount().disable();
	}
}
