//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package de.frankmuenster.hibiscus.mahoe.santander.assistant;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateAccountException;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateAccountAssistant;
import de.frankmuenster.hibiscus.mahoe.santander.SantanderBank;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.hbci.HBCI;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * BOS Konto-Anlagen-Assistent
 *
 * @author Frank Mnster
 *
 */
public class SantanderCreateAccountAssistant implements CreateAccountAssistant {

	/**
	 * Standardkonstruktor
	 */
	public SantanderCreateAccountAssistant() {
		// Absichtlich leer
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.assistant.CreateAccountAssistant#createAccount()
	 */
	@Override
	public Konto createAccount() throws CreateAccountException, OperationCanceledException {
		final SantanderCreateAccountDialog infoDialog = new SantanderCreateAccountDialog();
		SantanderCreateInfo info = null;
		try {
			info = infoDialog.open();
		} catch (OperationCanceledException e) { // NOPMD
			throw e;
		} catch (Exception e) { // NOPMD
			Logger.warn(I18N.tr("SantanderCreateAccountAssistant.openDialogFailedLog", e.getMessage())); //$NON-NLS-1$
			throw new CreateAccountException(I18N.tr("SantanderCreateAccountAssistant.openDialogFailed"), e); //$NON-NLS-1$
		}
		if (StringUtils.isEmpty(info.getLogin()) || StringUtils.isEmpty(info.getKtoNummer())) {
			throw new CreateAccountException(I18N.tr("SantanderCreateAccountAssistant.missingInputParameters")); //$NON-NLS-1$
		}
		return this.createKonto(info);

	}

	private Konto createKonto(final SantanderCreateInfo info) throws CreateAccountException {

		HBCIDBService dbService = null;
		try {
			dbService = (HBCIDBService) Application.getServiceFactory().lookup(HBCI.class, "database"); //$NON-NLS-1$
		} catch (Exception e) { // NOPMD
			Logger.error(I18N.tr("SantanderCreateAccountAssistant.noDbAccess"), e); //$NON-NLS-1$
			throw new CreateAccountException(I18N.tr("SantanderCreateAccountAssistant.noDbAccess"), e); //$NON-NLS-1$
		}
		Konto konto;
		try {
			konto = dbService.createObject(Konto.class, null);

			konto.setName(info.getOwner());
			konto.setBezeichnung(info.getDescription());

			if (info.getKtoType().equals(KontoType.GIRO)) {
				konto.setKontonummer(info.getKtoNummer());
			} else {
				konto.setKontonummer(info.getKtoNummer().substring(6, 15));
			}

			konto.setKundennummer(info.getLogin());
			konto.setUnterkonto(StringUtils.EMPTY);
			konto.setBLZ(SantanderBank.BLZ);
			konto.setAccountType(info.getKtoType().getValue());
			konto.setFlags(Konto.FLAG_NONE);
			konto.setKommentar(info.getComment());

			konto.setBackendClass(MahoeSynchronizeBackend.BACKEND_NAME);
			konto.store();

			// Setze META Daten
			konto.setMeta(MahoeSynchronizeBackend.BANK_NAME, info.getBankType().getIdString());
			if (info.getKtoType().equals(KontoType.GIRO)) {
				konto.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, SantanderBank.KTO_VER_GIRO);
				konto.setMeta(MahoeSynchronizeBackend.CARD_NUMBER, StringUtils.EMPTY);
				konto.setMeta(MahoeSynchronizeBackend.CARD_COMPANY, StringUtils.EMPTY);
				// Ausgleichbuchung
				konto.setMeta(MahoeSynchronizeBackend.CREATE_CLEARING, "NEIN"); //$NON-NLS-1$
				konto.setMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT, StringUtils.EMPTY);

			} else {
				konto.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, SantanderBank.KTO_VER_KREDIT);
				konto.setMeta(MahoeSynchronizeBackend.CARD_NUMBER, info.getKtoNummer());
				konto.setMeta(MahoeSynchronizeBackend.CARD_COMPANY, StringUtils.EMPTY);
				// Ausgleichbuchung
				final String createClearing = info.isCreateClearing() ? "JA" : "NEIN"; //$NON-NLS-1$ //$NON-NLS-2$
				konto.setMeta(MahoeSynchronizeBackend.CREATE_CLEARING, createClearing);
				konto.setMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT, info.getClearingAccount().getID());

			}
			// Kontoauszugebehandlung
			final String savePdf = info.isSavePDF() ? "JA" : "NEIN"; //$NON-NLS-1$ //$NON-NLS-2$
			konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF, savePdf);
			konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF_PATH, info.getSaveFolder());

			// Sync-Options setzen
			final SynchronizeOptions sync = new SynchronizeOptions(konto);
			sync.setAutoSaldo(false);
			sync.setSyncAuslandsUeberweisungen(false);
			sync.setSyncKontoauszuege(true);
			sync.setSyncOffline(false);
			sync.setSyncSaldo(true);
			sync.setSyncSepaDauerauftraege(false);
			sync.setSyncSepaLastschriften(false);
		} catch (RemoteException | ApplicationException e) {
			Logger.error(I18N.tr("SantanderCreateAccountAssistant.createAccountFailedLog", //$NON-NLS-1$
					e.getMessage()), e);
			throw new CreateAccountException(I18N.tr("SantanderCreateAccountAssistant.createAccountFailed"), e); //$NON-NLS-1$
		}

		return konto;

	}

}
