//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 
package de.frankmuenster.hibiscus.mahoe.santander.web;

import java.io.IOException;
import java.nio.charset.Charset;
import java.rmi.RemoteException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.Page;
import com.gargoylesoftware.htmlunit.WebResponse;
import com.gargoylesoftware.htmlunit.html.DomElement;
import com.gargoylesoftware.htmlunit.html.HtmlAnchor;
import com.gargoylesoftware.htmlunit.html.HtmlElement;
import com.gargoylesoftware.htmlunit.html.HtmlForm;
import com.gargoylesoftware.htmlunit.html.HtmlInput;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlSelect;
import com.gargoylesoftware.htmlunit.html.HtmlTable;
import com.gargoylesoftware.htmlunit.html.HtmlTableRow;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaverException;
import de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung fr ein Tagesgeldkonto.
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.ExcessiveImports" })
public final class SantanderWebSiteGiro
        extends AbstractSantanderWebSite<SantanderWebAccountInformation, SantanderBankMessages> {

    /**
     * 
     */
    private static final String TAGESGELDKONTEN = "Tagesgeldkonten"; //$NON-NLS-1$

    /**
     * CSV Felderanzahl
     */
    private static final int CSV_FIELDS = 33;

    /**
     * Konstruktor fr ein Tagesgeldkonto.
     * 
     * @param konto
     * @param userName
     * @throws WebSiteException
     */
    public SantanderWebSiteGiro(final Konto konto, final String userName) throws WebSiteException {
        super(konto, userName);
    }

    /**
     * Liefert die weiteren Kontodetails
     * 
     * @return die Kontodetails
     * @throws WebSiteException
     */
    @Override
    public SantanderWebAccountInformation getAccountInformation() throws WebSiteException {
        HtmlPage overviewOPage;
        try {
            overviewOPage = this.webClient.getPage(this.accountInfo.getBankType().getOverviewUrl());
            final String ktoNummer = this.accountInfo.getKonto().getKontonummer();
            HtmlAnchor details = null;
            if (overviewOPage.asText().contains(SantanderWebSiteGiro.TAGESGELDKONTEN)) { // $NON-NLS-1$
                final List<DomElement> tables = overviewOPage.getElementsByTagName(AbstractWebSite.TABLE_TAG);
                HtmlTable table = null;
                for (final DomElement domElement : tables) {
                    table = (HtmlTable) domElement;
                    Logger.debug(table.getFirstChild().getNextSibling().getTextContent());
                    if (SantanderWebSiteGiro.TAGESGELDKONTEN
                            .equals(table.getFirstChild().getNextSibling().getTextContent())) {
                        final List<HtmlTableRow> rows = table.getBodies().get(0).getRows();
                        for (final HtmlTableRow row : rows) {
                            if (row.getCell(0).asText().toLowerCase().contains(ktoNummer)) { // NOPMD
                                details = (HtmlAnchor) row.getCell(3).getFirstElementChild();
                                break;
                            }
                        }
                    }
                }
            }
            if (details == null) {
                throw new WebSiteException(I18N.tr("SantanderWebSiteGiro.detailsPageNotFound"), //$NON-NLS-1$
                        WebSiteExceptionTypes.PAGE_NOT_FOUND);
            } else {
                final HtmlPage detailsPage = details.click();
                // Logger.debug(detailsPage.asXml());
                // IBAN
                final HtmlElement iban = (HtmlElement) detailsPage
                        .getByXPath("/html/body/div/div[3]/div[1]/div/div[1]/div/div[5]/span[2]").get(0); //$NON-NLS-1$
                this.accountInfo.setIban(iban.asText());
                // BIC
                final HtmlElement bic = (HtmlElement) detailsPage
                        .getByXPath("/html/body/div/div[3]/div[1]/div/div[1]/div/div[6]/span[2]").get(0); //$NON-NLS-1$
                this.accountInfo.setBic(bic.asText());
                // Saldo
                final HtmlElement saldo = (HtmlElement) detailsPage
                        .getByXPath("/html/body/div/div[3]/div[1]/div/div[1]/div/div[7]/span[2]").get(0); //$NON-NLS-1$
                this.accountInfo.setSaldo(Utils.convertStringToDouble(saldo.asText()));
            }
        } catch (FailingHttpStatusCodeException | IOException e) {
            Logger.error(I18N.tr("SantanderWebSiteGiro.readAccountOverviewFailedLog", //$NON-NLS-1$
                    e.getMessage()), e);
            throw new WebSiteException(I18N.tr("SantanderWebSiteGiro.readAccountOverviewFailed"), // NOPMD //$NON-NLS-1$
                    WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
        }
        return this.accountInfo;
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.santander.web.AbstractSantanderWebSite#getBookedTransactions()
     */
    @Override
    public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {
        List<Umsatz> transactions = new ArrayList<>();
        try {
            final HtmlPage overviewOPage = this.webClient.getPage(this.accountInfo.getBankType().getOverviewUrl());
            final String ktoNummer = this.accountInfo.getKonto().getKontonummer();
            HtmlAnchor anchor = null;
            if (overviewOPage.asText().contains(SantanderWebSiteGiro.TAGESGELDKONTEN)) { // $NON-NLS-1$
                final List<DomElement> tables = overviewOPage.getElementsByTagName(AbstractWebSite.TABLE_TAG);
                HtmlTable table = null;
                for (final DomElement domElement : tables) {
                    table = (HtmlTable) domElement;
                    Logger.debug(table.getFirstChild().getNextSibling().getTextContent());
                    if (SantanderWebSiteGiro.TAGESGELDKONTEN
                            .equals(table.getFirstChild().getNextSibling().getTextContent())) {
                        final List<HtmlTableRow> rows = table.getBodies().get(0).getRows();
                        for (final HtmlTableRow row : rows) {
                            if (row.getCell(0).asText().toLowerCase(Locale.GERMAN).contains(ktoNummer)) {
                                anchor = (HtmlAnchor) row.getCell(0).getFirstElementChild().getFirstElementChild();
                                transactions = this.getTransactions(anchor);
                                break;
                            }
                        }
                    }
                }
            }
            if (anchor == null) {
                throw new WebSiteException(I18N.tr("SantanderWebSiteGiro.transactionsPageNotFound"), //$NON-NLS-1$
                        WebSiteExceptionTypes.PAGE_NOT_FOUND);
            }
        } catch (FailingHttpStatusCodeException | IOException e) {
            Logger.error(I18N.tr("SantanderWebSiteGiro.readBookFailedLog", e.getMessage())); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("SantanderWebSiteGiro.readBookFailed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.HTTP_STATUS_CODE, e);
        }
        return transactions;
    }

    /**
     * @param anchor
     * @return eine Liste mit den gefundenen Ums&auml;tzen
     * @throws IOException
     */
    private List<Umsatz> getTransactions(final HtmlAnchor anchor) throws IOException {
        final List<Umsatz> transactions = new ArrayList<>();
        final HtmlPage transPage = anchor.click();
        if (transPage.asText().contains("(Einklappen / Ausklappen)")) { //$NON-NLS-1$
            final HtmlForm formSearch = transPage.getFormByName("accInfoReg"); //$NON-NLS-1$
            // select last 180 days
            final HtmlSelect timeperiodSelect = formSearch.getSelectByName("accInfoReg.days"); //$NON-NLS-1$
            timeperiodSelect.setSelectedAttribute("-180", true); //$NON-NLS-1$
            final HtmlInput submitShow = formSearch.getInputByName("accInfoReg.linkButton.events.0"); //$NON-NLS-1$
            // Invoke Click
            final HtmlPage pageList = submitShow.click();
            final List<HtmlAnchor> anchors = pageList.getAnchors();
            HtmlAnchor anchorExport = null;
            for (int j = 0; j < anchors.size(); j++) {
                anchorExport = anchors.get(j);
                if ("export".equalsIgnoreCase(anchorExport.getTextContent())) { //$NON-NLS-1$
                    final Page csvExport = anchorExport.click();
                    final WebResponse response = csvExport.getWebResponse();
                    Logger.debug("ContentType: " + response.getContentType()); //$NON-NLS-1$
                    final String responseAsString = response.getContentAsString(Charset.forName("ISO-8859-1")); //$NON-NLS-1$
                    // Split in lines
                    final String[] lines = responseAsString.split("\n"); //$NON-NLS-1$
                    for (final String line : lines) {
                        final String[] transaction = line.split(";", -1); //$NON-NLS-1$
                        if (transaction.length == SantanderWebSiteGiro.CSV_FIELDS) {
                            if (transaction[0].equals("Buchungsdatum")) { //$NON-NLS-1$ //
                                                                          // NOPMD
                                continue;
                            }
                            final Umsatz umsatz = SantanderWebSiteGiro.loadLine(transaction);
                            transactions.add(umsatz);
                        }
                    }
                }
            }
        }
        return transactions;
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.santander.web.AbstractSantanderWebSite#getPreBookedTransactions()
     */
    @Override
    public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {
        // Gibt es bei Tagesgeld nicht
        return new ArrayList<>();
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite#getStartdate()
     */
    @Override
    protected Calendar getStartdate() {
        // Setzte das startDatum auf jetzt minus 180 Tage
        final Calendar startCalendar = Calendar.getInstance();
        startCalendar.add(Calendar.DAY_OF_YEAR, -180);
        return startCalendar;
    }

    /**
     * L&auml;d eine Zeile aus der CSV-Datei in einen Umsatz.
     * 
     * @param transaction
     *            die Umsatzzeile aus dem CSV
     * @return den Umsatz
     * @throws RemoteException
     */
    private static Umsatz loadLine(final String[] transaction) // NOPMD
            throws RemoteException {
        // Buchungsdatum;Wertstellung;Transaktionsart;Betrag (EUR);Saldo (EUR);
        // Verwendungszweck 1;Verwendungszweck 2;Verwendungszweck
        // 3;Verwendungszweck 4;
        // Verwendungszweck 5;Verwendungszweck 6;Verwendungszweck
        // 7;Verwendungszweck 8;
        // Verwendungszweck 9;Verwendungszweck 10;Verwendungszweck
        // 11;Verwendungszweck 12;
        // Verwendungszweck 13;Verwendungszweck 14;Verwendungszweck 15;
        final HBCIDBService dbService = Settings.getDBService();
        final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", //$NON-NLS-1$
                Locale.getDefault());
        final Umsatz umsatz = dbService.createObject(Umsatz.class, null);
        // Buchungsdatum
        Date date = null;
        try {
            date = sdf.parse(transaction[0]);
        } catch (final ParseException e) {
            Logger.error(I18N.tr("SantanderWebSiteGiro.parseBookingDateFailed", e.getMessage()), e); //$NON-NLS-1$
        }
        umsatz.setDatum(date);
        // Valuta
        try {
            date = sdf.parse(transaction[1]);
        } catch (final ParseException e) {
            Logger.error(I18N.tr("SantanderWebSiteGiro.parseValutaDateFailed", e.getMessage()), e); //$NON-NLS-1$
        }
        umsatz.setValuta(date);
        // Transaktionsart
        umsatz.setArt(transaction[2]);
        // Betrag in EUR
        umsatz.setBetrag(Utils.convertStringToDouble(transaction[3]));
        // Saldo
        if (StringUtils.isNotEmpty(transaction[4])) {
            umsatz.setSaldo(Utils.convertStringToDouble(transaction[4]));
        }
        // Zweck
        final StringBuilder zweck = new StringBuilder();
        for (int i = 5; i < 19; i++) {
            zweck.append(transaction[i]).append('\n');
        }
        zweck.append(transaction[19]);
        Utils.loadUmsatz(umsatz, zweck.toString());
        return umsatz;
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.santander.web.AbstractSantanderWebSite#savePDF(de.frankmuenster.hibiscus.mahoe.santander.web.SantanderWebAccountInformation)
     */
    @Override
    public int savePDF(final SantanderWebAccountInformation accountInfo) throws PdfSaverException {
        Logger.info(I18N.tr("AbstractSantanderWebSite.saveSheetsNotSupported")); //$NON-NLS-1$
        return 0;
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.santander.web.AbstractSantanderWebSite#isSupporting(de.willuhn.jameica.hbci.rmi.KontoType)
     */
    @Override
    protected boolean isSupporting(final KontoType type) {
        return KontoType.GIRO.equals(type);
    }
}
