//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.targobank.assistant;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateKontoDialog;
import de.frankmuenster.hibiscus.mahoe.assistant.AccountInfo;
import de.frankmuenster.hibiscus.mahoe.utils.DebitCardValidator;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.DirectoryInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.FormTextPart;

/**
 * Dialog zum Abfragen der MahoeMain Kreditkarteninformationen
 * 
 * @author Frank Mnster
 * 
 */
public class TargoBankCreateAccountDialog extends AbstractCreateKontoDialog<TargoBankCreateInfo> {

	/**
	 * Kartennummer Input
	 */
	private transient TextInput cardNumber;

	/**
	 * Kartenkonto Input
	 */
	private transient TextInput cardAccount;

	/**
	 * Anbieter Input
	 */
	private transient TextInput cardCompany;

	/**
	 * Login Input
	 */
	private transient TextInput login;

	/**
	 * Auszge speichern Input
	 */
	private transient CheckboxInput savePDF;

	/**
	 * Speicherordner Input
	 */
	private transient DirectoryInput saveFolder;

	/**
	 */
	public TargoBankCreateAccountDialog() {
		super();
		this.setTitle(I18N.tr("TargoBankCreateAccountDialog.dialogTitle")); //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected void paint(final Composite parent) {

		super.paint(parent);

		cardNumber = new TextInput(StringUtils.EMPTY, 20, "5500 0000 0000 0004"); //$NON-NLS-1$
		cardNumber.setMandatory(true);
		cardNumber.setComment(I18N.tr("TargoBankCreateAccountDialog.cardNumberComment")); //$NON-NLS-1$

		cardAccount = new TextInput(StringUtils.EMPTY, 20, "5500 0000 0000 0004"); //$NON-NLS-1$
		cardAccount.setComment(I18N.tr("TargoBankCreateAccountDialog.cardAccountComment")); //$NON-NLS-1$

		cardCompany = new TextInput(StringUtils.EMPTY, 50, "Select MasterCard"); //$NON-NLS-1$
		cardCompany.setMandatory(true);
		cardCompany.setComment(I18N.tr("TargoBankCreateAccountDialog.companyComment")); //$NON-NLS-1$

		login = new TextInput(StringUtils.EMPTY, 50, "Kundenkennung"); //$NON-NLS-1$
		login.setMandatory(true);
		login.setComment(I18N.tr("TargoBankCreateAccountDialog.loginComment")); //$NON-NLS-1$

		savePDF = new CheckboxInput(true);
		saveFolder = new DirectoryInput(StringUtils.EMPTY);

		container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblCardNumber"), cardNumber); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblCardAccount"), cardAccount); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblCardCompany"), cardCompany); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblLogin"), login); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblsavePdf"), savePDF); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblsavePdfFolder"), saveFolder); //$NON-NLS-1$

		container.addText(" ", true); //$NON-NLS-1$
		final StringBuffer formString = new StringBuffer(300)// NOPMD
				.append("<form><p>") //$NON-NLS-1$
				.append(I18N.tr("TargoBankCreateAccountDialog.websiteHint")) //$NON-NLS-1$
				.append("<br/><a href=\"http://progs.frankmuenster.de/mahoe/targo-bank\">http://progs.frankmuenster.de/mahoe/targo-bank</a><br/>") //$NON-NLS-1$
				.append("</p></form>"); //$NON-NLS-1$

		final FormTextPart text = new FormTextPart();
		text.setText(formString.toString());
		container.addPart(text);

		setSize(610, 460);

	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
	 */
	@Override
	protected TargoBankCreateInfo getData() {

		final AccountInfo info = super.getData();

		data.setOwner(info.getOwner());
		data.setComment(info.getComment());

		return data;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateKontoDialog#prepareData()
	 */
	@Override
	protected boolean prepareData() {

		boolean dataOK = true;

		data = new TargoBankCreateInfo();
		data.setCardNumber((String) cardNumber.getValue());
		data.setCardAccount((String) cardAccount.getValue());
		if (StringUtils.isEmpty(data.getCardAccount())) {
			data.setCardAccount(data.getCardNumber());
		}
		data.setCompany((String) cardCompany.getValue());
		data.setLogin((String) login.getValue());
		data.setSavePDF((Boolean) savePDF.getValue());
		data.setSaveFolder((String) saveFolder.getValue());

		if (StringUtils.isEmpty(data.getCardNumber())
				|| !DebitCardValidator.isValidateCardNumber(data.getCardNumber())) {
			dataOK = false;
			cardNumber.setComment(I18N.tr("TargoBankCreateAccountDialog.cardNumberWrong")); //$NON-NLS-1$
		} else {
			cardNumber.setComment(StringUtils.EMPTY);
		}
		if (StringUtils.isEmpty(data.getCompany())) {
			dataOK = false;
			cardCompany.setComment(I18N.tr("TargoBankCreateAccountDialog.cardTypeWrong")); //$NON-NLS-1$
		} else {
			cardCompany.setComment(StringUtils.EMPTY);
		}
		if (StringUtils.isEmpty(data.getLogin())) {
			dataOK = false;
			login.setComment(I18N.tr("TargoBankCreateAccountDialog.loginWrong")); //$NON-NLS-1$
		} else {
			login.setComment(StringUtils.EMPTY);
		}

		return dataOK;

	}
}
