//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.targobank.web;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.web.AbstractWebAccountInformation;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Bean mit Kontoinformationen von der Webseite
 * 
 * @author Frank Mnster
 * @version 1.0
 */
public class TargoBankWebAccountInformation extends AbstractWebAccountInformation {

	/**
	 * Das Kartenkonto
	 */
	private String cardAccount;

	/**
	 * Der Kartenanbieter
	 */
	private String cardCompany;

	/**
	 * Die Kartennummer
	 */
	private String cardNumber;

	/**
	 * Nchster Flligkeitstermin
	 */
	private Date nextPayDay;

	/**
	 * Der Verfgungsrahmen
	 */
	private Double verfuegungsrahmen;

	/**
	 * Standardkonstruktor
	 */
	public TargoBankWebAccountInformation() {
		super();
	}

	/**
	 * @return the cardAccount
	 */
	public String getCardAccount() {
		return cardAccount;
	}

	/**
	 * @return the cardCompany
	 */
	public String getCardCompany() {
		return cardCompany;
	}

	/**
	 * @return the cardNumber
	 */
	public String getCardNumber() {
		return cardNumber;
	}

	/**
	 * @return the nextPayDay
	 */
	public Date getNextPayDay() {
		return nextPayDay;
	}

	/**
	 * @return the verfuegungsrahmen
	 */
	public Double getVerfuegungsrahmen() {
		return verfuegungsrahmen;
	}

	/**
	 * @param cardAccount
	 *            the cardAccount to set
	 */
	public void setCardAccount(final String cardAccount) {
		this.cardAccount = cardAccount;
	}

	/**
	 * @param cardCompany
	 *            the cardCompany to set
	 */
	public void setCardCompany(final String cardCompany) {
		this.cardCompany = cardCompany;
	}

	/**
	 * @param cardNumber
	 *            the cardNumber to set
	 */
	public void setCardNumber(final String cardNumber) {
		this.cardNumber = cardNumber;
	}

	/**
	 * @param nextPayDay
	 *            the nextPayDay to set
	 */
	public void setNextPayDay(final Date nextPayDay) {
		this.nextPayDay = nextPayDay;
		updateComment();

	}

	/**
	 * @param verfuegungsrahmen
	 *            the verfuegungsrahmen to set
	 */
	public void setVerfuegungsrahmen(final Double verfuegungsrahmen) {
		this.verfuegungsrahmen = verfuegungsrahmen;
		updateComment();
	}

	private void updateComment() {

		final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", //$NON-NLS-1$
				Locale.GERMAN);

		final String rahmen = verfuegungsrahmen == null ? StringUtils.EMPTY : Double.toString(verfuegungsrahmen);
		final String payDay = nextPayDay == null ? StringUtils.EMPTY : sdf.format(nextPayDay);

		final String kommentar = I18N.tr("TargoBankSynchronizeJobKontoauszug.comment", //$NON-NLS-1$
				rahmen, payDay);

		setComment(kommentar);

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.AbstractWebAccountInformation#saveBankSpecificInfo(de.willuhn.util.ProgressMonitor)
	 */
	@Override
	protected void saveBankSpecificInfo(final ProgressMonitor monitor) throws ApplicationException {
		// Noch nix zu tun
	}
}
