//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.utils;

import java.io.IOException;
import java.io.OutputStream;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;

import de.willuhn.logging.Logger;

/**
 * Die &Uuml;bersetzer Klasse. Eigenst&auml;endig u.a. wegen JUNIT Tests
 * 
 * @author Frank Mnster
 * @version 1.0
 */
@SuppressWarnings("PMD.ShortClassName")
public final class I18N {

	/**
	 * Resource Pfad zu den Sprachdateien
	 */
	private static final String BUNDLE_NAME = "lang.mahoe"; //$NON-NLS-1$

	/**
	 * Resource Pfad zu den Sprachdateien
	 */

	/**
	 * Das Resource Bundle mit der Sprachdatei
	 */
	private static ResourceBundle bundle;

	/**
	 * Das Resource Bundle mit der Sprachdatei
	 */
	private static ResourceBundle bundleFallBack;

	/**
	 * Speichert alle nicht bersetzten Texte
	 */
	private static Properties unresolved = new Properties();

	/**
	 * Die Locale
	 */
	private static Locale locale = Locale.getDefault();

	static {
		final Locale localeDe = new Locale("de", "DE"); //$NON-NLS-1$ //$NON-NLS-2$
		try {
			bundle = ResourceBundle.getBundle(BUNDLE_NAME, locale);
			bundleFallBack = ResourceBundle.getBundle(BUNDLE_NAME, localeDe);
		} catch (MissingResourceException e) {
			// Wir laden die deutsche Sprachdatei
			locale = localeDe;
			bundle = ResourceBundle.getBundle(BUNDLE_NAME, locale);

			// Die Zeile ist nur fr den Eclipse Externalize String wizard
			// Das dort angegebenen 'Resource File' wird benutzt
			@SuppressWarnings("unused")
			final ResourceBundle toUse = ResourceBundle.getBundle("lang.mahoe_de_DE", locale); //$NON-NLS-1$
		}
	}

	private I18N() {
		// Absichtlich leer
	}

	/**
	 * Liefert die bersetzung in der aktuellen Local
	 * 
	 * @param key
	 *            das Schlsselwort
	 * 
	 * @return die bersetzung in der aktuellen Local
	 */
	public static String tr(final String key) { // NOPMD
		String result;
		try {
			result = bundle.getString(key);
		} catch (MissingResourceException e) {
			try {
				// Versuche mit deutscher Sprachdatei
				result = bundleFallBack.getString(key);
			} catch (MissingResourceException e1) {
				// result = '!' + key + '!';
				result = key;
			}
			unresolved.put(key, result);
		}
		return result;
	}

	/**
	 * Liefert die bersetzung in der aktuellen Local. Bitte zustzlich noch
	 * Variablen im Text an ({0}, etc.), die nach der bersetzung eingefgt
	 * werden.
	 * 
	 * @param key
	 *            das Schlsselwort
	 * @param replacements
	 *            die Ersetzungen
	 * 
	 * @return die bersetzung mit den Ersetzungen in der aktuellen Local
	 */
	public static String tr(final String key, final String... replacements) { // NOPMD
		return MessageFormat.format(tr(key), (Object[]) replacements);
	}

	/**
	 * Liefert die bersetzung in der aktuellen Local. Bitte zustzlich noch
	 * eine Variable im Text an ({0}), die nach der bersetzung eingefgt wird.
	 * 
	 * @param key
	 *            das Schlsselwort
	 * @param replacement
	 *            die Ersetzungen
	 * 
	 * @return die bersetzung mit der Ersetzung in der aktuellen Local
	 */
	public static String tr(final String key, final String replacement) { // NOPMD
		return tr(key, new String[] { replacement });
	}

	/**
	 * Schreibt alle bis dato nicht uebersetzbaren Strings in den angegebenen
	 * OutputStream.
	 * 
	 * @param oStream
	 *            Stream, in den geschrieben werden soll.
	 * @throws IOException
	 */
	public static void storeUntranslated(final OutputStream oStream) throws IOException {
		Logger.info(I18N.tr("I18N.storeMissingTranslations")); //$NON-NLS-1$
		unresolved.store(oStream, "unresolved strings for locale " + locale.toString()); //$NON-NLS-1$
	}

}
